/* ======== psychoviewer_mm ========
* Copyright (C) 2004-2005 sslice
* No warranties of any kind
*
* License: zlib/libpng
*
* Author(s): James "sslice" Gray
* ============================
*/

#ifndef _INCLUDE_PsychoViewer_H
#define _INCLUDE_PsychoViewer_H

#include <ISmmPlugin.h>
#include <sourcehook/sourcehook.h>
#include <igameevents.h>
#include <sh_vector.h>

using namespace SourceHook;

#define PLUGIN_VERSION	"1.2"

class WebLink {
public:
	WebLink()
	{
		Keyword = new char [1];
		URL = new char [1];
	}
	~WebLink() {
		delete [] Keyword;
		delete [] URL;
	}
	void SetKeyword(const char *word) {
		delete [] Keyword;
		Keyword = new char [Q_strlen(word)+1];
		Q_strcpy(Keyword,word);
	}
	void SetURL(const char *url) {
		delete [] URL;
		URL = new char [Q_strlen(url)+1];
		Q_strcpy(URL,url);
	}
	char *GetKeyword() {
		char *word = new char [Q_strlen(Keyword)+1];
		Q_strcpy(word,Keyword);
		return word;
	}
	char *GetURL() {
		char *url = new char [Q_strlen(URL)+1];
		Q_strcpy(url,URL);
		return url;
	}
private:
	char *Keyword;
	char *URL;
};
class PsychoViewer : public ISmmPlugin, public IMetamodListener
{
public:
	bool Load(PluginId id, ISmmAPI *ismm, char *error, size_t maxlen, bool late);
	bool Unload(char *error, size_t maxlen);
	void AllPluginsLoaded();
	bool Pause(char *error, size_t maxlen)
	{
		return true;
	}
	bool Unpause(char *error, size_t maxlen)
	{
		return true;
	}
public:
	int GetApiVersion() { return PLAPI_VERSION; }
public:
	const char *GetAuthor()
	{
		return "sslice";
	}
	const char *GetName()
	{
		return "PsychoViewer";
	}
	const char *GetDescription()
	{
		return "Ingame web browser originally intended to view PsychoStats";
	}
	const char *GetURL()
	{
		return "http://www.pdxg.net/";
	}
	const char *GetLicense()
	{
		return "zlib/libpng";
	}
	const char *GetVersion()
	{
		return PLUGIN_VERSION;
	}
	const char *GetDate()
	{
		return __DATE__;
	}
	const char *GetLogTag()
	{
		return "PSYCHOVIEWER";
	}
public:
	//These functions are from IServerPluginCallbacks
	//Note, the parameters might be a little different to match the actual calls!

	////Called on LevelInit.  Server plugins only have pMapName
	//bool LevelInit(const char *pMapName, char const *pMapEntities, char const *pOldLevel, char const *pLandmarkName, bool loadGame, bool background);

	//Called on ServerActivate.  Same definition as server plugins
	void ServerActivate(edict_t *pEdictList, int edictCount, int clientMax);

	////Called on a game tick.  Same definition as server plugins
	//void GameFrame(bool simulating);

	////Called on level shutdown.  Same definition as server plugins 
	//void LevelShutdown(void);

	////Client is activate (whatever that means).  We get an extra parameter...
	//// "If bLoadGame is true, don't spawn the player because its state is already setup."
	//void ClientActive(edict_t *pEntity, bool bLoadGame);

	////Client disconnects - same as server plugins
	//void ClientDisconnect(edict_t *pEntity);

	////Client is put in server - same as server plugins
	//void ClientPutInServer(edict_t *pEntity, char const *playername);

	//Sets the client index - same as server plugins
	void SetCommandClient(int index);

	////Called on client settings changed (duh) - same as server plugins
	//void ClientSettingsChanged(edict_t *pEdict);

	////Called on client connect.  Unlike server plugins, we return whether the 
	//// connection is allowed rather than set it through a pointer in the first parameter.
	//// You can still supercede the GameDLL through RETURN_META_VALUE(MRES_SUPERCEDE, true/false)
	//bool ClientConnect(edict_t *pEntity, const char *pszName, const char *pszAddress, char *reject, int maxrejectlen);

	//Called when a client uses a command.  Unlike server plugins, it's void.
	// You can still supercede the gamedll through RETURN_META(MRES_SUPERCEDE).
	//void ClientCommand(edict_t *pEntity);

	////From IMetamodListener
	//virtual void OnLevelShutdown();

	//bool FireGameEvent(IGameEvent *event, bool bDontBroadcast);

	//Pop up PsychoStats window given an edict
    void ShowWebLink(edict_t *pEntity, const char *url);
	int UserMessageIndex(const char *messageName);
	void LoadWebLinks();

	//Hooks say/say_team command
	void ChatHook();


	CVector<WebLink*> vWebLinks;
	ICvar * m_ICvar;
	ConCommand *pSayCmd;
	ConCommand *pTeamSayCmd;
	//IGameEventManager2 *m_GameEventManager;	
	IVEngineServer *m_Engine;
	IServerGameDLL *m_ServerDll;
	IServerGameClients *m_ServerClients;
	SourceHook::CallClass<IVEngineServer> *m_Engine_CC;
};

extern PsychoViewer g_PsychoViewer;
PLUGIN_GLOBALVARS();

void UTIL_TrimRight(char *buffer);
void UTIL_TrimLeft(char *buffer);

#endif //_INCLUDE_PsychoViewer_H
